@@ -30,7 +30,7 @@
  * @param mi A #MpdObj
  * @param artist an artist name
  *
- * Grab's a list of albums of a certain artist from mpd.
+ * Grabs a list of albums of a certain artist from mpd.
  * if artist is %NULL it grabs all albums
  *
  * @returns A #MpdData list.
@@ -41,7 +41,7 @@ MpdData *	mpd_database_get_albums		(MpdObj *mi, char *artist);
 /**
  * @param mi a #MpdObj
  *
- * returns a list of all availible artists.
+ * returns a list of all available artists.
  *
  * @returns a #MpdData list
  */
@@ -51,7 +51,7 @@ MpdData *	mpd_database_get_artists		(MpdObj *mi);
 /**
  * @param mi a #MpdObj
  *
- * Get's the complete datababse, only returns songs
+ * Gets the complete database, only returns songs
  *
  * @returns a #MpdData list with songs
  */
@@ -63,55 +63,13 @@ MpdData * mpd_database_get_complete(MpdObj *mi);
  *@param mi A #MpdObj
  *@param path The path mpd should update.
  *
- * Force mpd to update (parts of )the database.
+ * Force mpd to update (parts of) the database.
  *
  * @returns a #MpdError
  */
 int 	mpd_database_update_dir		(MpdObj *mi, char *path);
 
 
-
-/**
- * @param mi A #MpdObj
- * @param string The search string
- *
- * client side search function with best "match" option..
- * It splits the search string into tokens. (on the ' ')  every token is then matched using regex.
- * It not tokenize between [()].
- *
- * So f.e. Murder Hooker|Davis  matches songs where title/filename/artist/album contains murder and hooker or murder and davis in any order.
- *
- * Warning: This function can be slow.
- *
- * @returns a #MpdData list
- */
-MpdData *	mpd_database_token_find		(MpdObj *mi , char *string);
-
-
-
-/**
- * @param mi A #MpdObj
- * @param path path of the playlist
- *
- * Deletes a playlist.
- * @returns
- */
-
-int mpd_database_delete_playlist(MpdObj *mi,char *path);
-
-
-
-/**
- * @param mi a #MpdObj
- * @param name The name of the playlist
- *
- * Saves the current playlist to a file.
- *
- * @returns a #MpdError. #MPD_OK if succesfull,
- * #MPD_DATABASE_PLAYLIST_EXIST when the playlist allready exists.
- */ 
-int		mpd_database_save_playlist			(MpdObj *mi, char *name);
-
 /**
  * @param mi a #MpdObj
  * @param table table
@@ -132,17 +90,26 @@ MpdData * mpd_database_find(MpdObj *mi, int table, char *string, int exact);
  */
 MpdData * mpd_database_get_directory(MpdObj *mi,char *path);
 
-
 /**
  * @param mi a #MpdObj
- * @param playlist the playlist you need the content off.
+ * @param path a string containing the  path
  *
- * Only works with patched mpd.
- * Check for %mpd_server_command_allowed(mi, "listPlaylistInfo");
+ * Recursively list all the songs directory path
  *
- * @returns a #MpdData list
+ * returns a #MpdData
  */
-MpdData *mpd_database_get_playlist_content(MpdObj *mi,char *playlist);
+MpdData * mpd_database_get_directory_recursive(MpdObj *mi, const char *path);
+
+/**
+ * @param mi A #MpdObj
+ * @param path an Path to a file
+ *
+ * Grabs the song info for a single file. Make sure you pass a url to a song
+ * and not a directory, that might result in strange behaviour.
+ *
+ * @returns a #mpd_Song
+ */
+mpd_Song * mpd_database_get_fileinfo(MpdObj *mi,const char *path);
 
 /*@}*/
 
@@ -151,16 +118,7 @@ MpdData *mpd_database_get_playlist_content(MpdObj *mi,char *playlist);
  * The following functions provide an interface to the improved search capabilities of mpd 0.12.0. 
  */
 /*@{*/
-/**
- * @param mi A #MpdObj
- * @param path an Path to a file
- *
- * Grabs the song info for a single file. Make sure you pass an url to a song
- * and not a directory, that might result in strange behauviour.
- *
- * @returns a #mpd_Song
- */
-mpd_Song * mpd_database_get_fileinfo(MpdObj *mi,const char *path);
+
 
 /**
  * @param mi A #MpdObj
@@ -202,7 +160,7 @@ void mpd_database_search_start(MpdObj *mi, int exact);
  * @param mi a #MpdObj
  * @param field a #mpd_TagItems
  *
- * Starts a field search, f.e. if you want a list of all albums, you do;
+ * Starts a field search, eg. if you want a list of all albums, you do;
  * 
  * @code 
  * mpd_database_search_field_start(mi, MPD_TAG_ITEM_ALBUM);
@@ -225,33 +183,146 @@ void mpd_database_search_field_start(MpdObj *mi, mpd_TagItems field);
  *
  * Commits the search and gathers the result in a #MpdData list.
  *
- * @returns a #MpdData list with the search result,or NULL when nothing is found
+ * @returns a #MpdData list with the search result, or NULL when nothing is found
  */
 MpdData * mpd_database_search_commit(MpdObj *mi);
 
 
 
 /*@}*/
-
-/** \defgroup searchdeprecated Deprecated database search functions
+/** \defgroup databaseSearchStats Database Search Statistics
  * \ingroup database
- * The following functions are deprecated
+ * A extension to the database search, that instead of returning the full results.
+ * Only reports back a few statistics about the result. 
+ * For now Number of Songs are reported and total playtime.
  */
 /*@{*/
+
+/*! \var typedef mpd_SearchStat MpdDBStats 
+    \brief A Structure containing numberOfSongs and playTime 
+    
+    see #mpd_SearchStats
+*/
+
+typedef mpd_SearchStats MpdDBStats;
+
 /**
- * @param mi a #MpdObj
- * @param field The table field
- * @returns a #MpdData
- */
-MpdData *	mpd_database_get_unique_tags		(MpdObj *mi, int field,...) __attribute__((deprecated));
-/**
- * @param mi a #MpdObj
- * @param exact 1 for exact search 0 for fuzzy
+ * @param mi A #MpdObj
  *
- * WARNING: This function is deprecated, use mpd_database_search_start 
- * @returns a #MpdData
+ * Starts a search, you can add constraints by calling #mpd_database_search_add_constraint.
+ * To get the result call #mpd_database_search_stats_commit
+ * 
+ * This function requires mpd 0.13.0 or higher 
  */
-MpdData *	mpd_database_find_adv		(MpdObj *mi,int exact, ...) __attribute__((deprecated));
+
+void mpd_database_search_stats_start(MpdObj *mi);
+
+
+/**
+ * @param mi A #MpdObj
+ *
+ * Gets statistics results of a search.
+ * 
+ * return a #MpdDBStats
+ */
+MpdDBStats * mpd_database_search_stats_commit(MpdObj *mi);
+
+/**
+ * @param data a #MpdDBStats
+ *
+ * frees the #MpdDBStats structure.
+ */
+void mpd_database_search_free_stats(MpdDBStats *data);
+
 /*@}*/
 
+/** \defgroup databasePlaylist Database Playlist
+ * \ingroup database
+ */
+/*@{*/
+
+
+/**
+ * @param mi A #MpdObj
+ * @param path path of the playlist
+ *
+ * Deletes a playlist.
+ * @returns
+ */
+
+int mpd_database_delete_playlist(MpdObj *mi,char *path);
+
+/**
+ * @param mi a #MpdObj
+ * @param name The name of the playlist
+ *
+ * Saves the current playlist to a file.
+ *
+ * @returns a #MpdError. #MPD_OK if successful,
+ * #MPD_DATABASE_PLAYLIST_EXIST when the playlist already exists.
+ */ 
+int		mpd_database_save_playlist			(MpdObj *mi, char *name);
+
+/**
+ * @param mi a #MpdObj
+ * @param playlist the playlist you need the content of.
+ *
+ * Needs mpd 0.12.0 or higher.
+ *
+ * @returns a #MpdData list
+ */
+MpdData *mpd_database_get_playlist_content(MpdObj *mi,char *playlist);
+
+/**
+ * @param mi a #MpdObj
+ * @param path a string contains the path of the playlist
+ * @param file a string contains the path of the song to add
+ *
+ * Add a path to a stored playlist.
+ * Needs 0.13.0
+ */
+void mpd_database_playlist_list_add(MpdObj *mi, const char *path, const char *file);
+
+/**
+ * @param mi a #MpdObj
+ * @param path a string containing the path of the playlist
+ * @param pos an int representing the position of a song
+ *
+ * Deletes the song at position pos from a playlist.
+ * Needs mpd 0.13.0
+ *
+ */
+void mpd_database_playlist_list_delete(MpdObj *mi, const char *path, int pos);
+
+/**
+ * @param mi a #MpdObj
+ * @param path a string containing the path of the playlist
+ *
+ * Clears the content of a stored playlist, also used to create an empty playlist
+ * Needs mpd 0.13.0
+ */
+void mpd_database_playlist_clear(MpdObj *mi,const char *path);
+
+/**
+ * @param mi a #MpdObj
+ * @param old a string, old playlist name
+ * @param new  a string, new playlist name
+ * 
+ * Renames a stored playlist
+ * Needs mpd 0.13.0
+ */
+void mpd_database_playlist_rename(MpdObj *mi, const char *old_name, const char *new_name);
+
+/**
+ * @param mi a #MpdObj
+ * @param playlist a string containing the path of the playlist
+ * @param old_pos an integer representing old position
+ * @param new_pos an integer representing the position to move old_pos to.
+ *
+ * Moves songs in a stored playlist
+ * Needs mpd 0.13.0
+ */
+int mpd_database_playlist_move(MpdObj *mi, const char *playlist, int old_pos, int new_pos);
+MpdData * mpd_database_playlist_list(MpdObj *mi);
+/*@}*/
 #endif
