@@ -19,10 +19,10 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <assert.h>
 #define __USE_GNU
 
 #include <string.h>
-#include <regex.h>
 #include <stdarg.h>
 #include <config.h>
 #include "debug_printf.h"
@@ -63,6 +63,7 @@ int mpd_status_update(MpdObj *mi)
 	}
 
 
+
 	if(mi->status != NULL)
 	{
 		mpd_freeStatus(mi->status);
@@ -106,24 +107,18 @@ int mpd_status_update(MpdObj *mi)
 
         if(mi->CurrentState.playlistLength == mi->status->playlistLength)
         {
-            what_changed |= MPD_CST_SONGID; 
+//            what_changed |= MPD_CST_SONGID; 
         }
 		/* save new id */
 		mi->CurrentState.playlistid = mi->status->playlist;
 	}
-	if(mi->CurrentState.queueid != mi->status->playlistqueue)
-	{
-		/* set MPD_CST_QUEUE to be changed */
-		what_changed |= MPD_CST_QUEUE;
-
-		/* save new id */
-		mi->CurrentState.queueid = mi->status->playlistqueue;
-	}
+	
 	if(mi->CurrentState.storedplaylistid != mi->status->storedplaylist)
 	{
 		/* set MPD_CST_QUEUE to be changed */
 		what_changed |= MPD_CST_STORED_PLAYLIST;
 
+
 		/* save new id */
 		mi->CurrentState.storedplaylistid = mi->status->storedplaylist;
 	}
@@ -242,8 +237,53 @@ int mpd_status_update(MpdObj *mi)
 		mi->CurrentState.updatingDb = mi->status->updatingDb;
 	}
 
+
     mi->CurrentState.playlistLength = mi->status->playlistLength;
 
+
+    /* Detect changed outputs */
+    if(!mi->has_idle)
+    {
+        if(mi->num_outputs >0 )
+        {
+            mpd_OutputEntity *output = NULL;
+            mpd_sendOutputsCommand(mi->connection);
+            while (( output = mpd_getNextOutput(mi->connection)) != NULL)
+            {	
+                if(mi->output_states[output->id] != output->enabled)
+                {
+                    mi->output_states[output->id] = output->enabled;
+                    what_changed |= MPD_CST_OUTPUT;
+                }
+                mpd_freeOutputElement(output);
+            }
+            mpd_finishCommand(mi->connection);
+        }
+        else
+        {
+            /* if no outputs, lets fetch them */
+            mpd_server_update_outputs(mi);
+            if(mi->num_outputs == 0)
+            {
+                assert("No outputs defined? that cannot be\n");
+            }
+            what_changed |= MPD_CST_OUTPUT;
+        }
+    }else {
+        char *name;
+        mpd_sendGetEventsCommand(mi->connection);    
+        while((name = mpd_getNextEvent(mi->connection))){
+            if(strcmp(name, "output") == 0){
+                what_changed |= MPD_CST_OUTPUT;
+            }else if (strcmp(name, "stored_playlist")==0) {
+                what_changed |= MPD_CST_STORED_PLAYLIST;
+            }
+            free(name);
+       }
+       mpd_finishCommand(mi->connection);
+    }
+
+  
 	/* Run the callback */
 	if((mi->the_status_changed_callback != NULL) && what_changed)
 	{
