@@ -22,7 +22,6 @@
 #define __USE_GNU
 
 #include <string.h>
-#include <regex.h>
 #include <stdarg.h>
 #include <config.h>
 #include <glib.h>
@@ -34,6 +33,7 @@
 static void mpd_free_queue_ob(MpdObj *mi);
 static void mpd_server_free_commands(MpdObj *mi);
 
+
 char *libmpd_version = LIBMPD_VERSION;
 #ifndef HAVE_STRNDUP
 /**
@@ -66,7 +66,6 @@ char * strndup(const char *s, size_t n)
 static void mpd_init_MpdServerState(MpdServerState *state)
 {
 	state->playlistid 	= -1;
-	state->queueid 		= -2;
 	state->storedplaylistid = -1;
 	state->state 		= -1;
 	state->songid 		= -1;     	
@@ -148,6 +147,10 @@ static MpdObj * mpd_create()
 	/* no need to initialize, but set it to anything anyway*/
 	mi->search_field = MPD_TAG_ITEM_ARTIST;
 
+    /* outputs */
+    mi->num_outputs = 0;
+    mi->output_states = NULL;
+    mi->has_idle = 0;
 	/* commands */
 	mi->commands = NULL;
 	return mi;
@@ -215,7 +218,13 @@ int mpd_check_error(MpdObj *mi)
 	mi->error = mi->connection->error;
 	mi->error_mpd_code = mi->connection->errorCode;
 	/*TODO: do I need to strdup this? */
-	mi->error_msg = strdup(mi->connection->errorStr);
+    if(!g_utf8_validate(mi->connection->errorStr, -1, NULL)){
+        mi->error_msg = g_locale_to_utf8(mi->connection->errorStr, -1, NULL, NULL,NULL);
+    }else{
+        mi->error_msg = g_strdup(mi->connection->errorStr);
+    }
+
+    if(mi->error_msg == NULL) mi->error_msg = g_strdup("Failed to convert error message to utf-8");
 
 	/* Check for permission */
 	/* First check for an error reported by MPD
@@ -229,7 +238,15 @@ int mpd_check_error(MpdObj *mi)
 		if (mi->the_error_callback)
 		{
             debug_printf(DEBUG_ERROR, "Error callback 1 (ACK)");
-			mi->the_error_callback(mi, mi->error_mpd_code, mi->error_msg, mi->the_error_signal_userdata );
+			if(mi->the_error_callback(mi, mi->error_mpd_code, mi->error_msg, mi->the_error_signal_userdata ))
+            {
+                debug_printf(DEBUG_ERROR, "Error callback told me to disconnect");
+                mpd_disconnect(mi);
+                free(mi->error_msg);
+                mi->error_msg = NULL;
+
+                return MPD_SERVER_ERROR;
+            }
 		}
 		free(mi->error_msg);
 		mi->error_msg = NULL;
@@ -238,7 +255,7 @@ int mpd_check_error(MpdObj *mi)
 	if(mi->error)
 	{
 
-		debug_printf(DEBUG_ERROR, "Following error occured: %i: code: %i msg: %s", mi->error,mi->connection->errorCode, mi->error_msg);
+		debug_printf(DEBUG_ERROR, "Following error occurred: %i: code: %i msg: %s", mi->error,mi->connection->errorCode, mi->error_msg);
 
 		if (mi->the_error_callback)
 		{
@@ -483,6 +500,9 @@ int mpd_server_get_allowed_commands(MpdObj *mi)
 		mi->commands[num_commands-1].enabled = TRUE;
 		mi->commands[num_commands].command_name = NULL;
 		mi->commands[num_commands].enabled = FALSE;
+        if(strcmp(mi->commands[num_commands-1].command_name, "idle") == 0) {
+            mi->has_idle = TRUE;
+        }
 	}
 	mpd_finishCommand(mi->connection);
 	mpd_sendNotCommandsCommand(mi->connection);
@@ -534,7 +554,6 @@ int mpd_disconnect(MpdObj *mi)
 		mi->CurrentSong = NULL;
 	}
 	mi->CurrentState.playlistid = -1;
-	mi->CurrentState.queueid = -2;
 	mi->CurrentState.storedplaylistid = -1;
 	mi->CurrentState.state = -1;
 	mi->CurrentState.songid = -1;
@@ -557,9 +576,12 @@ int mpd_disconnect(MpdObj *mi)
 	mi->search_type = MPD_SEARCH_TYPE_NONE;
 	/* no need to initialize, but set it to anything anyway*/
 	mi->search_field = MPD_TAG_ITEM_ARTIST;
-
-
-
+    /* outputs */
+    mi->num_outputs = 0;
+    if(mi->output_states)
+        g_free(mi->output_states);
+    mi->output_states = NULL;
+    mi->has_idle = 0;
 	
 	memcpy(&(mi->OldState), &(mi->CurrentState) , sizeof(MpdServerState));
 
@@ -652,11 +674,23 @@ int mpd_connect_real(MpdObj *mi,mpd_Connection *connection)
     {
         return retv;
     }
+/*
+
+    retv = mpd_server_update_outputs(mi);
+    if(retv != MPD_OK)
+        return retv;
+*/
+    debug_printf(DEBUG_INFO,  "Propagating connection changed");
+
     if(mi->the_connection_changed_callback != NULL)
 	{
 		mi->the_connection_changed_callback( mi, TRUE, mi->the_connection_changed_signal_userdata );
 	}
-
+    retv = mpd_server_update_outputs(mi);
+    /*
+    if(retv != MPD_OK)
+        return retv;
+        */
 	debug_printf(DEBUG_INFO, "Connected to mpd");
 	return MPD_OK;
 }
@@ -724,6 +758,9 @@ MpdData *mpd_new_data_struct(void)
 	data->next = NULL;
 	data->prev = NULL;
     data->first = NULL;
+
+    data->userdata = NULL;
+    data->freefunc = NULL;
 	return (MpdData*)data;
 }
 
@@ -894,10 +931,15 @@ void mpd_data_free(MpdData *data)
         } else if(data_real->type == MPD_DATA_TYPE_DIRECTORY) {
             if(data_real->directory)free(data_real->directory);
         } else if(data_real->type == MPD_DATA_TYPE_PLAYLIST) {
-            if(data_real->playlist)free(data_real->playlist);				
+            if(data_real->playlist) mpd_freePlaylistFile(data_real->playlist);				
         } else {
             free((void*)(data_real->tag));
         }
+        if(data_real->freefunc)
+        {
+            if(data_real->userdata)
+                data_real->freefunc(data_real->userdata);
+        }
         data_real = data_real->next;
         g_slice_free1(sizeof(*temp), temp);
     }
@@ -1150,3 +1192,33 @@ int mpd_misc_get_tag_by_name(char *name)
     }
     return MPD_TAG_NOT_FOUND;
 }
+
+int mpd_server_update_outputs(MpdObj *mi)
+{
+    mpd_OutputEntity *output = NULL;
+    if(!mpd_check_connected(mi))
+    {
+        debug_printf(DEBUG_WARNING,"not connected\n");
+        return MPD_NOT_CONNECTED;
+    }
+    if(mpd_lock_conn(mi))
+    {
+        debug_printf(DEBUG_ERROR,"lock failed\n");
+        return MPD_LOCK_FAILED;
+    }        
+    mpd_sendOutputsCommand(mi->connection);
+    while (( output = mpd_getNextOutput(mi->connection)) != NULL)
+    {	
+        mi->num_outputs++;
+        mi->output_states = g_realloc(mi->output_states,mi->num_outputs*sizeof(int));
+        mi->output_states[mi->num_outputs-1] = FALSE;/*output->enabled;*/
+        mpd_freeOutputElement(output);
+    }
+    mpd_finishCommand(mi->connection);
+    return mpd_unlock_conn(mi);
+}
+
+int mpd_server_has_idle(MpdObj *mi)
+{
+    return mi->has_idle;
+}
