@@ -66,6 +66,10 @@
 #  endif
 #endif
 
+#ifndef WIN32
+#include <sys/un.h>
+#endif
+
 #ifndef MSG_DONTWAIT
 #  define MSG_DONTWAIT 0
 #endif
@@ -100,18 +104,21 @@ static int do_connect_fail(mpd_Connection *connection,
                            const struct sockaddr *serv_addr, int addrlen)
 {
 	int iMode = 1; /* 0 = blocking, else non-blocking */
+	if (connect(connection->sock, serv_addr, addrlen) == SOCKET_ERROR)
+		return 1;
 	ioctlsocket(connection->sock, FIONBIO, (u_long FAR*) &iMode);
-	return (connect(connection->sock,serv_addr,addrlen) == SOCKET_ERROR
-			&& WSAGetLastError() != WSAEWOULDBLOCK);
+	return 0;
 }
 #else /* !WIN32 (sane operating systems) */
 static int do_connect_fail(mpd_Connection *connection,
                            const struct sockaddr *serv_addr, int addrlen)
 {
-	int flags = fcntl(connection->sock, F_GETFL, 0);
+	int flags;
+	if (connect(connection->sock, serv_addr, addrlen) < 0)
+		return 1;
+	flags = fcntl(connection->sock, F_GETFL, 0);
 	fcntl(connection->sock, F_SETFL, flags | O_NONBLOCK);
-	return (connect(connection->sock,serv_addr,addrlen)<0 &&
-				errno!=EINPROGRESS);
+	return 0;
 }
 #endif /* !WIN32 */
 
@@ -129,7 +136,7 @@ static int mpd_connect(mpd_Connection * connection, const char * host, int port,
 	 * Setup hints
 	 */
 	hints.ai_flags     = AI_ADDRCONFIG;
-	hints.ai_family    = PF_UNSPEC;
+	hints.ai_family    = AF_UNSPEC;
 	hints.ai_socktype  = SOCK_STREAM;
 	hints.ai_protocol  = IPPROTO_TCP;
 	hints.ai_addrlen   = 0;
@@ -151,6 +158,8 @@ static int mpd_connect(mpd_Connection * connection, const char * host, int port,
 
 	for (res = addrinfo; res; res = res->ai_next) {
 		/* create socket */
+		if (connection->sock >= 0)
+			closesocket(connection->sock);
 		connection->sock = socket(res->ai_family, SOCK_STREAM,
 		                          res->ai_protocol);
 		if (connection->sock < 0) {
@@ -167,11 +176,13 @@ static int mpd_connect(mpd_Connection * connection, const char * host, int port,
 		/* connect stuff */
  		if (do_connect_fail(connection,
 		                    res->ai_addr, res->ai_addrlen)) {
- 			/* try the next address family */
+ 			/* try the next address */
  			closesocket(connection->sock);
  			connection->sock = -1;
  			continue;
 		}
+
+		break;
 	}
 
 	freeaddrinfo(addrinfo);
@@ -222,6 +233,8 @@ static int mpd_connect(mpd_Connection * connection, const char * host, int port,
 		break;
 	}
 
+	if (connection->sock >= 0)
+		closesocket(connection->sock);
 	if((connection->sock = socket(dest->sa_family,SOCK_STREAM,0))<0) {
 		strcpy(connection->errorStr,"problems creating socket");
 		connection->error = MPD_ERROR_SYSTEM;
@@ -306,7 +319,7 @@ void mpd_setConnectionTimeout(mpd_Connection * connection, float timeout) {
 }
 
 static int mpd_parseWelcome(mpd_Connection * connection, const char * host, int port,
-                            char * rt, char * output) {
+                            char * output) {
 	char * tmp;
 	char * test;
 	int i;
@@ -339,6 +352,53 @@ static int mpd_parseWelcome(mpd_Connection * connection, const char * host, int
 	return 0;
 }
 
+#ifndef WIN32
+static int mpd_connect_un(mpd_Connection * connection,
+			  const char * host, float timeout)
+{
+	int error, flags;
+	size_t path_length;
+	struct sockaddr_un saun;
+
+	path_length = strlen(host);
+	if (path_length >= sizeof(saun.sun_path)) {
+		strcpy(connection->errorStr, "unix socket path is too long");
+		connection->error = MPD_ERROR_UNKHOST;
+		return -1;
+	}
+
+	saun.sun_family = AF_UNIX;
+	memcpy(saun.sun_path, host, path_length + 1);
+
+	connection->sock = socket(AF_UNIX, SOCK_STREAM, 0);
+	if (connection->sock < 0) {
+		strcpy(connection->errorStr, "problems creating socket");
+		connection->error = MPD_ERROR_SYSTEM;
+		return -1;
+	}
+
+	mpd_setConnectionTimeout(connection, timeout);
+
+	flags = fcntl(connection->sock, F_GETFL, 0);
+	fcntl(connection->sock, F_SETFL, flags | O_NONBLOCK);
+
+	error = connect(connection->sock, (struct sockaddr*)&saun, sizeof(saun));
+	if (error < 0) {
+		/* try the next address family */
+		close(connection->sock);
+		connection->sock = 0;
+
+		snprintf(connection->errorStr,MPD_BUFFER_MAX_LENGTH,
+			 "problems connecting to \"%s\": %s",
+			 host, strerror(errno));
+		connection->error = MPD_ERROR_CONNPORT;
+		return -1;
+	}
+
+	return 0;
+}
+#endif /* WIN32 */
+
 mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	int err;
 	char * rt;
@@ -347,6 +407,7 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	struct timeval tv;
 	fd_set fds;
 	strcpy(connection->buffer,"");
+	connection->sock = -1;
 	connection->buflen = 0;
 	connection->bufstart = 0;
 	strcpy(connection->errorStr,"");
@@ -361,7 +422,13 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	if (winsock_dll_error(connection))
 		return connection;
 
-	if (mpd_connect(connection, host, port, timeout) < 0)
+#ifndef WIN32
+	if (host[0] == '/')
+		err = mpd_connect_un(connection, host, timeout);
+	else
+#endif
+		err = mpd_connect(connection, host, port, timeout);
+	if (err < 0)
 		return connection;
 
 	while(!(rt = strstr(connection->buffer,"\n"))) {
@@ -409,7 +476,7 @@ mpd_Connection * mpd_newConnection(const char * host, int port, float timeout) {
 	strcpy(connection->buffer,rt+1);
 	connection->buflen = strlen(connection->buffer);
 
-	if(mpd_parseWelcome(connection,host,port,rt,output) == 0) connection->doneProcessing = 1;
+	if(mpd_parseWelcome(connection,host,port,output) == 0) connection->doneProcessing = 1;
 
 	free(output);
 
@@ -448,9 +515,9 @@ static void mpd_executeCommand(mpd_Connection * connection, char * command) {
 	FD_SET(connection->sock,&fds);
 	tv.tv_sec = connection->timeout.tv_sec;
 	tv.tv_usec = connection->timeout.tv_usec;
-
-	while((ret = select(connection->sock+1,NULL,&fds,NULL,&tv)==1) ||
+    while((ret = select(connection->sock+1,NULL,&fds,NULL,&tv)==1) ||
 			(ret==-1 && SELECT_ERRNO_IGNORE)) {
+        fflush(NULL);
 		ret = send(connection->sock,commandPtr,commandLen,MSG_DONTWAIT);
 		if(ret<=0)
 		{
@@ -467,7 +534,6 @@ static void mpd_executeCommand(mpd_Connection * connection, char * command) {
 
 		if(commandLen<=0) break;
 	}
-
 	if(commandLen>0) {
 		perror("");
 		snprintf(connection->errorStr,MPD_ERRORSTR_MAX_LENGTH,
@@ -673,8 +739,7 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 	status->repeat = 0;
 	status->random = 0;
 	status->playlist = -1;
-	status->playlistqueue = -1;
-	status->storedplaylist = -1;
+    status->storedplaylist = -1;
 	status->playlistLength = -1;
 	status->state = -1;
 	status->song = 0;
@@ -707,12 +772,6 @@ mpd_Status * mpd_getStatus(mpd_Connection * connection) {
 		else if(strcmp(re->name,"playlist")==0) {
 			status->playlist = strtol(re->value,NULL,10);
 		}
-		else if(strcmp(re->name,"playlistqueue")==0) {
-			status->playlistqueue = strtol(re->value,NULL,10);
-		}
-		else if (strcmp(re->name, "storedplaylist") == 0 ) {
-			status->storedplaylist = strtol(re->value,NULL,10);
-		}
 		else if(strcmp(re->name,"playlistlength")==0) {
 			status->playlistLength = atoi(re->value);
 		}
@@ -947,6 +1006,7 @@ static void mpd_finishSong(mpd_Song * song) {
 	if(song->date) free(song->date);
 	if(song->genre) free(song->genre);
 	if(song->composer) free(song->composer);
+	if(song->performer) free(song->performer);
 	if(song->disc) free(song->disc);
 	if(song->comment) free(song->comment);
 }
@@ -976,6 +1036,7 @@ mpd_Song * mpd_songDup(mpd_Song * song) {
 	if(song->date) ret->date = strdup(song->date);
 	if(song->genre) ret->genre= strdup(song->genre);
 	if(song->composer) ret->composer= strdup(song->composer);
+	if(song->performer) ret->performer = strdup(song->performer);
 	if(song->disc) ret->disc = strdup(song->disc);
 	if(song->comment) ret->comment = strdup(song->comment);
 	ret->time = song->time;
@@ -1017,10 +1078,12 @@ mpd_Directory * mpd_directoryDup(mpd_Directory * directory) {
 
 static void mpd_initPlaylistFile(mpd_PlaylistFile * playlist) {
 	playlist->path = NULL;
+    playlist->mtime = NULL;
 }
 
 static void mpd_finishPlaylistFile(mpd_PlaylistFile * playlist) {
 	if(playlist->path) free(playlist->path);
+    if(playlist->mtime) free(playlist->mtime);
 }
 
 mpd_PlaylistFile * mpd_newPlaylistFile(void) {
@@ -1040,6 +1103,7 @@ mpd_PlaylistFile * mpd_playlistFileDup(mpd_PlaylistFile * playlist) {
 	mpd_PlaylistFile * ret = mpd_newPlaylistFile();
 
 	if(playlist->path) ret->path = strdup(playlist->path);
+    if(playlist->mtime) ret->mtime = strdup(playlist->mtime);
 
 	return ret;
 }
@@ -1138,9 +1202,17 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 
 		if(entity->type == MPD_INFO_ENTITY_TYPE_SONG &&
 				strlen(re->value)) {
-			if(!entity->info.song->artist &&
-					strcmp(re->name,"Artist")==0) {
-				entity->info.song->artist = strdup(re->value);
+			if(strcmp(re->name,"Artist")==0) {
+				if(entity->info.song->artist) {
+					int length = strlen(entity->info.song->artist);
+					entity->info.song->artist = realloc(entity->info.song->artist, 
+					                                    length + strlen(re->value) + 3);
+					strcpy(&((entity->info.song->artist)[length]), ", ");
+					strcpy(&((entity->info.song->artist)[length + 2]), re->value);
+				}
+				else {
+					entity->info.song->artist = strdup(re->value);
+				}
 			}
 			else if(!entity->info.song->album &&
 					strcmp(re->name,"Album")==0) {
@@ -1176,15 +1248,40 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 			}
 			else if(!entity->info.song->genre &&
 					strcmp(re->name, "Genre") == 0) {
-				entity->info.song->genre = strdup(re->value);
+				if(entity->info.song->genre) {
+					int length = strlen(entity->info.song->genre);
+					entity->info.song->genre = realloc(entity->info.song->genre, 
+					                                   length + strlen(re->value) + 4);
+					strcpy(&((entity->info.song->genre)[length]), ", ");
+					strcpy(&((entity->info.song->genre)[length + 3]), re->value);
+				}
+				else {
+					entity->info.song->genre = strdup(re->value);
+				}
 			}
-			else if(!entity->info.song->composer &&
-					strcmp(re->name, "Composer") == 0) {
-				entity->info.song->composer = strdup(re->value);
+			else if(strcmp(re->name, "Composer") == 0) {
+				if(entity->info.song->composer) {
+					int length = strlen(entity->info.song->composer);
+					entity->info.song->composer = realloc(entity->info.song->composer, 
+					                                      length + strlen(re->value) + 3);
+					strcpy(&((entity->info.song->composer)[length]), ", ");
+					strcpy(&((entity->info.song->composer)[length + 2]), re->value);
+				}
+				else {
+					entity->info.song->composer = strdup(re->value);
+				}
 			}
-			else if(!entity->info.song->performer &&
-					strcmp(re->name, "Performer") == 0) {
-				entity->info.song->performer = strdup(re->value);
+			else if(strcmp(re->name, "Performer") == 0) {
+				if(entity->info.song->performer) {
+					int length = strlen(entity->info.song->performer);
+					entity->info.song->performer = realloc(entity->info.song->performer, 
+					                                       length + strlen(re->value) + 3);
+					strcpy(&((entity->info.song->performer)[length]), ", ");
+					strcpy(&((entity->info.song->performer)[length + 2]), re->value);
+				}
+				else {
+					entity->info.song->performer = strdup(re->value);
+				}
 			}
 			else if(!entity->info.song->disc &&
 					strcmp(re->name, "Disc") == 0) {
@@ -1198,6 +1295,10 @@ mpd_InfoEntity * mpd_getNextInfoEntity(mpd_Connection * connection) {
 		else if(entity->type == MPD_INFO_ENTITY_TYPE_DIRECTORY) {
 		}
 		else if(entity->type == MPD_INFO_ENTITY_TYPE_PLAYLISTFILE) {
+            if(!entity->info.playlistFile->mtime &&
+                    strcmp(re->name, "Last-Modified") == 0) {
+                    entity->info.playlistFile->mtime = strdup(re->value);
+            }
 		}
 
 		mpd_getNextReturnElement(connection);
@@ -1961,25 +2062,21 @@ void mpd_sendPlaylistDeleteCommand(mpd_Connection *connection,
 	free(sPlaylist);
 	free(string);
 }
-
-void mpd_sendQueueInfoCommand(mpd_Connection * connection) {
-	mpd_sendInfoCommand(connection,"queueinfo\n");
-}
-
-void mpd_sendDequeueCommand(mpd_Connection * connection, int songPos) {
-	int len = strlen("dequeue")+2+INTLEN+3;
-	char *string = malloc(len);
-	snprintf(string, len, "dequeue \"%i\"\n", songPos);
-	mpd_executeCommand(connection,string);
-	free(string);
-}
-void mpd_sendQueueIdCommand(mpd_Connection * connection, int songId) {
-	int len = strlen("queueid")+2+INTLEN+3;
-	char *string = malloc(len);
-	snprintf(string, len, "queueid \"%i\"\n", songId);
-	mpd_executeCommand(connection,string);
-	free(string);
-}
 void mpd_sendClearErrorCommand(mpd_Connection * connection) {
 	mpd_executeCommand(connection,"clearerror\n");
 }
+
+
+void mpd_sendGetEventsCommand(mpd_Connection *connection) {
+    mpd_executeCommand(connection, "idle\nnoidle\n");
+//    mpd_executeCommand(connection, "noidle\n");
+}
+
+char * mpd_getNextEvent(mpd_Connection *connection)
+{
+    return mpd_getNextReturnElementNamed(connection, "changed");
+}
+
+void mpd_sendListPlaylistsCommand(mpd_Connection * connection) {
+    mpd_sendInfoCommand(connection, "listplaylists\n");
+}
